@@ -4,7 +4,7 @@
 #' functions in the \pkg{shiny} package. The former is used to create a
 #' container for table, and the latter is used in the server logic to render the
 #' table.
-#' @inheritParams shiny::dataTableOutput
+#' @param outputId output variable to read the table from
 #' @param width the width of the table container
 #' @param height the height of the table container
 #' @param fill passed to \code{htmlwidgets::\link{shinyWidgetOutput}()}, see
@@ -42,7 +42,6 @@ DTOutput = dataTableOutput
 
 #' @export
 #' @rdname dataTableOutput
-#' @inheritParams shiny::renderDataTable
 #' @param expr an expression to create a table widget (normally via
 #'   \code{\link{datatable}()}), or a data object to be passed to
 #'   \code{datatable()} to create a table widget
@@ -53,17 +52,30 @@ DTOutput = dataTableOutput
 #'   browsers to slow down or crash. Note that if you want to use
 #'   \code{renderDataTable} with \code{shiny::bindCache()}, this must be
 #'   \code{FALSE}.
+#' @param env The parent environment for the reactive expression. By default,
+#'   this is the calling environment, the same as when defining an ordinary
+#'   non-reactive expression. If \code{expr} is a quosure and \code{quoted} is
+#'   \code{TRUE}, then \code{env} is ignored.
+#' @param quoted If it is \code{TRUE}, then the \code{\link{quote}()}ed value of
+#'   \code{expr} will be used when \code{expr} is evaluated. If \code{expr} is a
+#'   quosure and you would like to use its expression as a value for
+#'   \code{expr}, then you must set \code{quoted} to \code{TRUE}.
 #' @param funcFilter (for expert use only) passed to the \code{filter} argument
 #'   of \code{\link{dataTableAjax}()}
-#' @param future whether the server-side filter function should be executed
-#'   as a future or as a standard synchronous function. If true, the future
-#'   will be evaluated according to the session's \link[future]{plan}.
+#' @param future whether the server-side filter function should be executed as a
+#'   future or as a standard synchronous function. If true, the future will be
+#'   evaluated according to the session's \link[future]{plan}.
+#' @param outputArgs A list of arguments to be passed through to the implicit
+#'   call to \code{\link{dataTableOutput}()} when
+#'   \code{\link{renderDataTable}()} is used in an interactive R Markdown
+#'   document.
 #' @param ... ignored when \code{expr} returns a table widget, and passed as
 #'   additional arguments to \code{\link{datatable}()} when \code{expr} returns
 #'   a data object
 renderDataTable = function(
     expr, server = TRUE, env = parent.frame(), quoted = FALSE,
-    funcFilter = dataTablesFilter, future = FALSE, ...
+    funcFilter = dataTablesFilter, future = FALSE,
+    outputArgs = list(), ...
   ) {
   if (!quoted) expr = substitute(expr)
 
@@ -86,7 +98,7 @@ renderDataTable = function(
   outputInfoEnv[["session"]] = NULL
 
   exprFunc = shiny::exprToFunction(expr, env, quoted = TRUE)
-  argFunc = shiny::exprToFunction(list(..., server = server), env, quoted = FALSE)
+  argFunc = shiny::exprToFunction(list(..., server = server), parent.frame(), quoted = FALSE)
   widgetFunc = function() {
     opts = options(DT.datatable.shiny = TRUE); on.exit(options(opts), add = TRUE)
     instance = exprFunc()
@@ -169,9 +181,10 @@ renderDataTable = function(
       uiFunc = dataTableOutput,
       renderFunc = function(shinysession, name, ...) {
         domain = tempVarsPromiseDomain(outputInfoEnv, outputName = name, session = shinysession)
-        removeTimestampFromSnapshot(name)
+        removeTimestampFromSnapshot(name, shinysession)
         promises::with_promise_domain(domain, renderFunc())
       },
+      outputArgs = outputArgs,
       cacheHint = cacheHint
     )
   } else {
@@ -181,7 +194,8 @@ renderDataTable = function(
         domain = tempVarsPromiseDomain(outputInfoEnv, outputName = name, session = shinysession)
         removeTimestampFromSnapshot(name)
         promises::with_promise_domain(domain, renderFunc())
-      }
+      },
+      outputArgs = outputArgs
     )
   }
 
@@ -216,11 +230,11 @@ setAll = function(lst, env) {
   invisible()
 }
 
-removeTimestampFromSnapshot = function(name) {
+removeTimestampFromSnapshot = function(name, session) {
   shiny::snapshotPreprocessInput(paste0(name, "_state"), function(value) {
     value$time <- NULL
     value
-  })
+  }, session)
 }
 
 # This promise domain is needed to set/unset temporary variables in
@@ -473,6 +487,9 @@ reloadData = function(
 #'   levels, and numeric columns should have the same or smaller range,
 #'   otherwise the filters may never be able to reach certain rows in the data,
 #'   unless you explicitly update the filters with \code{updateFilters()}.
+#'
+#'   If the \code{ColReorder} extension is used, the new \code{data} must have
+#'   column names that match the original data column names exactly.
 #' @export
 replaceData = function(proxy, data, ..., resetPaging = TRUE, clearSelection = 'all') {
   dataTableAjax(proxy$session, data, ..., outputId = proxy$rawId)
@@ -618,6 +635,9 @@ dataTablesFilter = function(data, params) {
   # index in data via its name because the two indices won't match when columns
   # are reordered via the colReorder extension
   imap = unlist(lapply(q$columns, function(col) {
+    # if data doesn't have column names (e.g., #1108), assume it's not reordered
+    # (which may not be true)
+    if (is.null(names(data))) return(0L)
     k = col[['name']]
     if (!is.character(k) || k == '') return(0L)
     i = match(k, names(data))
@@ -643,6 +663,12 @@ dataTablesFilter = function(data, params) {
   # start searching with all rows
   i = seq_len(n)
 
+  # apply SearchBuilder query if present
+  if (!is.null(s <- q$searchBuilder)) {
+    r = sbEvaluateSearch(s, data)
+    if (!is.null(r)) i = which(r)
+  }
+
   # search by columns
   if (length(i)) for (j in names(q$columns)) {
     col = q$columns[[j]]
